#!/usr/bin/env python
'''
This file contains Python command example for the RM module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

need to add description 
'''
import errno, sys ,struct ,socket
import colorsys

from python_sdk_api.sx_api import *

from test_infra_common import *

from optparse import OptionParser,Option


###################################################### 
#    defines
######################################################
hw_resources_out = ["TCAM ", "Linear", 
                    "KVD Hash", "KVD Linear",
                    "Router Adjacency", "Replication Matrix",
                    "ACL Group Table", "PGT",  "Flow Counter", "ACL Regions"]

logical_resources_out = [
    "UC MAC Table ",   
    "MC MAC Table ",  
    "FIB IPV4 UC Table ",
    "FIB IPV6 UC Table ",
    "FIB IPV4 MC Table ",
    "FIB IPV6 MC Table ",
    "ARP IPV4 Table ", 
    "ARP IPV6 Table ",
    "L3 MC REPLICATIONS Table",
    "Unicast Adjacency Table",
    "L2 MC VECTORS Table ",
    "LAG VECTORS Table ",
    "FlOOD VECTORS Table",
    "ACL Extended Actions Table ",
    "ACL PBS Table",
    "eRIF List ",
    "Tunnel RTDP",
    "ILM Table",
    "RPF Group Table",
    "VLAN Table",
    "VPorts Table",
    "FID Table",
    "VNI Table",
    "Tunnel IPv6 FDB Table",
    "MPLS NHLFE Table",
    "Policy Based MPLS ILM Table",
    "NVE IPv6 List Table",
    "ACL Regions",
    "RIPS IPv6 register",
    "IGMPv3 IPv4 Table",
    "IGMPv3 IPv6 Table",
    "Tunnel Decap IPv4 Table",
    "Tunnel Decap IPv6 Table",
    "ACL Rules 18B Key",
    "ACL Rules 32B Key",
    "ACL Rules 54B Key",
    "RIF Counter Basic",
    "RIF Counter Mixed",
    "RIF Counter Enhanced",
    "Flow Counter",
    "ACL GROUPS Table ",
    "Internal Type",
    "ACL RULES Table ",    
    "Decap rules",
    "IGMP V3 table",
    "A TCAM One-key Block Table",
    "A TCAM Two-keys Block Table",
    "A TCAM Four-keys Block Table",
    "A TCAM Six-keys Block Table",
    "FCF FORWORDING Table ",
    "RMID Manager Table ",
    "NVE MC List",
    "SMID Manager",
    "FIB IPV4 MC System Table ",
    "FIB IPV6 MC System Table ",
    "RIF Counter",
    ]




###################################################### 
#    Functions API
######################################################
   

def print_sdk_utilization(list_element, print_header=False):
    
    if print_header:
        print ("================================================================================================")
        header = ["Resource", "HW Table", "Logical Entries ", "HW Entries", "Utilization(%)"]
        print "|%25s|%15s|%15s|%15s|%15s|" % (header[0],header[1],header[2],header[3],header[4])
        print ("================================================================================================")

    if list_element.sdk_table_entries_used ==0 and list_element.sdk_table_entries_allocated==0:
        return
    print "|%25s|%15s|%15d|%15d|%15.1f|" % (logical_resources_out[list_element.resource], hw_resources_out[list_element.hw_table_type], list_element.sdk_table_entries_used,\
                 list_element.sdk_table_entries_allocated, list_element.hw_table_utilization_per_resource/10.0)                                                                                        
    
def print_sdk_free_entries_count (resource, free_count, print_header=False):
    if print_header:
        print ("============================================================")
        header = ["Resource", "Free Entries"]
        print "|%40s|%15s|" % (header[0],header[1])
        print ("============================================================")
    if (resource < len(logical_resources_out)):
        print "|%40s|%15s|" %(logical_resources_out[resource], free_count)


def get_hw_utilization(handle, resource=None):
    if resource==None:
        for resource in range(RM_HW_TABLE_TYPE_MIN_E, RM_HW_TABLE_TYPE_MAX_E + 1):
            utilization_p = new_uint32_t_p()
            rc = sx_api_rm_hw_utilization_get(handle,
                                                 resource,
                                                utilization_p)
            if rc != SX_STATUS_SUCCESS:
                continue
                
            utilization = uint32_t_p_value(utilization_p)
            print "Utilization for HW resource %s is %.1f" % (hw_resources_out[resource], utilization/10.0)
    else :
        utilization_p = new_uint32_t_p()
        rc = sx_api_rm_hw_utilization_get(handle,
                                                 resource,
                                                utilization_p)
        if rc != SX_STATUS_SUCCESS:
              print "HW Utilization failed with rc = [%d]" % (rc)
                
        utilization = uint32_t_p_value(utilization_p)
        print "Utilization for HW resource %s is %.1f" % (hw_resources_out[resource], utilization/10.0)
        
        
def get_logical_utilization(handle, resource_list_in=[]):
    resource_list = new_sx_api_table_type_t_arr(RM_SDK_TABLE_TYPE_MAX_E)
    utilization_list = new_sx_api_rm_table_utilization_t_arr(RM_SDK_TABLE_TYPE_MAX_E)
    list_count_p = new_uint32_t_p()
        
    if resource_list_in==[]:
        for resource in range(RM_SDK_TABLE_TYPE_MIN_E, RM_SDK_TABLE_TYPE_INTERNAL_E):    
            uint32_t_p_assign(list_count_p,1)        
            sx_api_table_type_t_arr_setitem(resource_list, 0, resource)
        
            rc = sx_api_rm_sdk_table_utilization_get(handle, resource_list, list_count_p, utilization_list)
            if rc != SX_STATUS_SUCCESS:                
                continue
            data_cnt = uint32_t_p_value(list_count_p)
            ut = sx_api_rm_table_utilization_t_arr_getitem(utilization_list,0)
            print_sdk_utilization(ut, resource==RM_SDK_TABLE_TYPE_MIN_E)
    else:
        uint32_t_p_assign(list_count_p,len(resource_list_in))
        for i, resource in enumerate(resource_list_in):                        
            sx_api_table_type_t_arr_setitem(resource_list, i, resource)
            
        rc = sx_api_rm_sdk_table_utilization_get(handle, resource_list, list_count_p, utilization_list)
        if rc != SX_STATUS_SUCCESS:
                print "Utilization get failed: [rc=%d]" % (rc)
                return
        data_cnt = uint32_t_p_value(list_count_p)
        for i in xrange(data_cnt):
            ut = sx_api_rm_table_utilization_t_arr_getitem(utilization_list,i)
            print_sdk_utilization(ut)
        

def get_logical_free_entries_count(handle, resource_list_in=[]):
    free_count_p = new_uint32_t_p()
    if resource_list_in==[]:
        for resource in range(RM_SDK_TABLE_TYPE_MIN_E, RM_SDK_TABLE_TYPE_INTERNAL_E):
            rc = sx_api_rm_free_entries_by_type_get(handle, resource, free_count_p);
            if rc == SX_STATUS_SUCCESS:
                free_count = uint32_t_p_value(free_count_p)
                print_sdk_free_entries_count(resource, free_count, resource==RM_SDK_TABLE_TYPE_MIN_E)
    else:
        print_header = True
        for resource in resource_list_in:                        
            rc = sx_api_rm_free_entries_by_type_get(handle, resource, free_count_p);
            if rc == SX_STATUS_SUCCESS:
                free_count = uint32_t_p_value(free_count_p)
                print_sdk_free_entries_count(resource, free_count, print_header)
                if print_header==True:
                    print_header = False
    delete_uint32_t_p(free_count_p)

